@@ -5,19 +5,11 @@ include Config
 pwd = Dir.pwd
 pwd.sub!(%r{[^/]+/[^/]+$}, "")
 
-language, extension = 'C', '_new'
-opaque = 'opaque'
+language, extension = 'C', '_new_trigger'
+opaque = 'language_handler'
 
 suffix = ARGV[1].to_s
 
-case ARGV[0].to_i
-when 70
-   language = 'newC'
-when 73, 74
-   extension = "_new_trigger"
-   opaque = 'language_handler'
-end
-
 begin
    f = File.new("test_setup.sql", "w")
    IO.foreach("test_setup#{extension}.sql") do |x| 
